% Base code to produce time series (Figure 2) or bifurcation diagrams
% (Figure 3) for the seasonal garlic mustard model

%% PARAMETERS
%%% Management parameters %%%
h1 = 0; % spring herbicide
p2 = 0; % pulling
c2 = 0; % early clipping
c3 = 0; % late clipping
h4 = 0; % fall herbicide
ws = 0; % biocontrol (seed effect)
wr = 0; % biocontrol (rosette effect)

%%% Demographic parameters (Table 1) %%%
v = 0.8228; % new seed survival
g1 = 0.55034; % new seed germination
g2 = 0.31705; % bank seed germination
s1 = 0.131; % seedling to 1st year rosette transition
s3 = 1; % 1st year rosette survival
s5 = 1; % 2nd year rosette to flowering adult transition
a = 1; % flowering to fruiting adult transition
r = 0.54; % flowering to resprouting adult transition (Pardini 2008)
fr = 0.95; % resprout fertility reduction (Pardini 2008)
decay_yrs = 10; % seed bank degradation time
    b1 = 1-1/(decay_yrs*12)*2; % bank seed survival
    b2 = 1-1/(decay_yrs*12)*1; 
    b3 = 1-1/(decay_yrs*12)*1; 
    b4 = 1-1/(decay_yrs*12)*8;
B0f = 7.48933; % density dependence constants 
    B1f = -0.03893;
    B0s = 0.11635;
    B1s = -0.01612;
    B2s = -0.00144;
    B3s = -0.00092;
    B0w = 1.32702;
    B1w = -0.50269;

%% INITIAL CONDITIONS
S_new0 = 0; % new seeds
S_bank0 = 100; % bank seeds
Sg0 = 0; % seedings
R0 = 0; % rosettes
A_flower0 = 0; % flowering adults
A_fruit0 = 0; % fruiting adults
A_resprout0 = 0; % reprouting adults

n = [S_new0 S_bank0 Sg0 R0 A_flower0 A_fruit0 A_resprout0];

%% SIMULATIONS
% To produce bifurcation diagrams, loop simulations over a range of values
% for the bifucation parameter, doing a large number of iterations (t_iter) 
% to reach asymptotic dynamics. For time series at set parameter values, 
% the "for loop" over different bifurcation parameter values and multiple 
% iterations below can be omitted.

values = 0:0.01:1; % values to vary bifurcation parameter over
t_iter = 3000; % number of iterations (seasons) to run
t_rec = 300; % additional iterations to record
    
for i = 1:length(values) % loop over all bifurcation parameter values
    h4 = values(i); % put desired bifurcation parameter here

    %% Management matrices
    %%% Season 1
    M1 = diag([1 1 1 1 1 1 1]);
    M1(3,3) = (1 - h1); % spring herbicide on seedlings
    M1(4,4) = (1 - h1); % spring herbicide on rosettes
    
    %%% Season 2
    M2 = diag([1 1 1 1 1 1 1]);
    M2(5,5) = (1 - p2)*(1 - c2); % pulling, clipping flowering adults
    M2(7,5) = c2*r; % resprouts from early clipping
    
    %%% Season 3
    M3 = diag([1 1 1 1 1 1 1]);
    M3(6,6) = (1 - c3); % clipping fruiting adults
    
    %%% Season 4
    M4 = diag([1 1 1 1 1 1 1]);
    M4(1,1) = (1-ws); % biocontrol on seeds
    M4(4,4) = (1-max(0,(0.895*ws-0.008)))*(1-h4); % biocontrol, fall herbicide on rosettes

    %% Run a year-long simulation
    for t = 1:4:t_iter + t_rec

        %%% Season 1 %%%
        n(t,:) = M1*n(t,:)'; % management for Season 1
        B1 = zeros(7); % projection matrix B1
            B1(4,3) = s1; B1(5,4) = s5; B1(2,2) = b1;
        Rt = n(t, 4);
            f = exp(B0f + B1f * Rt); % dd for fertility
        n(t+1,:) = B1*n(t,:)'; % transition to Season 2
        
        %%% Season 2 %%%
        n(t+1,:) = M2*n(t+1,:)'; % management for Season 2 
        Rt = n(t+1, 4); At = n(t+1, 5);
            s2 = 1/(1 + exp(-(B0s + B1s*At + B2s*Rt + B3s*At*Rt))); % dd for 1st yr rosette survival 
        B2 = zeros(7); % projection matrix B2
            B2(4,4) = s2; B2(6,5) = a; B2(2,2) = b2;
        n(t+2,:) = B2*n(t+1,:)'; % transition to Season 3
        
        %%% Season 3 %%%
        n(t+2,:) = M3*n(t+2,:)'; % management for Season 3
        B3 = zeros(7); % projection matrix B3
            B3(4,4) = s3; B3(1,6) = f; B3(1,7) = f*(1-fr); B3(2,2) = b3;
        n(t+3,:) = B3*n(t+2,:)'; % transition to Season 4
        
        %%% Season 4 %%%
        n(t+3,:) = M4*n(t+3,:)'; % management in Season 4
        Rt = n(t+3, 4);
            s4 = 1/(1 + exp(-(B0w + B1w * log(Rt + 1)))); % dd for 1st to 2nd yr rosette survival
        B4 = zeros(7); % projection matrix B4
            B4(4,4) = s4; B4(3,1) = v*g1; B4(2,1) = v*(1-g1); B4(3,2) = g2; B4(2,2) = b4*(1-g2);
        n(t+4,:) = B4*n(t+3,:)'; % transition to Season 1
        
    end

%% Partition the entries of "n" into different stages if desired.
% As in this example, we may only want to store densities after asymptotic
% dynamics have been reached (here, we are recording an additional t_rec
% seasons after the first t_iter seasons).

%S_new_eq = n(t_iter+1:t_iter + t_rec, 1);
%S_bank_eq = n(t_iter+1:t_iter + t_rec, 2);
%Sg_eq = n(t_iter+1:t_iter + t_rec, 3);
R_eq = n(t_iter+1:t_iter + t_rec, 4);
A_flower_eq = n(t_iter+1:t_iter + t_rec, 5);
A_fruit_eq = n(t_iter+1:t_iter + t_rec, 6);
A_resprout_eq = n(t_iter+1:t_iter + t_rec, 7);
totaleq = R_eq + A_flower_eq + A_fruit_eq + A_resprout_eq; % rosettes + all adults

%% Store seasonal densities separately if desired.
seasons = 4; co1 = 1; co2 = 1; co3 = 1; co4 = 1;

for k = 1:length(totaleq)
    if mod(k,seasons) == 1 
        season1(co1, i) = totaleq(k);
        co1 = co1 + 1;
    end
    
    if mod(k,seasons) == 2
        season2(co2, i) = totaleq(k); 
        co2 = co2 + 1;
    end
    
    if mod(k,seasons) == 3
        season3(co3, i) = totaleq(k);
        co3 = co3 + 1;
    end
    
    if mod(k,seasons) == 0
        season4(co4, i) = totaleq(k);
        co4 = co4 + 1;
    end
end

end

%% Plot seasonal densities if desired.
figure(1)
subplot(2, 2, 1)
plot(values, season1, 'k.'); 
title('Season 1')

subplot(2, 2, 2)
plot(values, season2, 'k.'); 
title('Season 2')

subplot(2, 2, 3)
plot(values, season3, 'k.');
title('Season 3')

subplot(2, 2, 4)
plot(values, season4, 'k.');
title('Season 4')