% A slightly modified version of gm_base.m that returns all the seasonal 
% population vectors in the seasonal 8-cycle (for the given management 
% parameters)

function [eq1 eq2 eq3 eq4] = gm_eq(h1, p2, c2, c3, h4, ws)

%% PARAMETERS
%%% Demographic parameters (Table 1) %%%
v = 0.8228; % new seed survival
g1 = 0.55034; % new seed germination
g2 = 0.31705; % bank seed germination
s1 = 0.131; % seedling to 1st year rosette transition
s3 = 1; % 1st year rosette survival
s5 = 1; % 2nd year rosette to flowering adult transition
a = 1; % flowering to fruiting adult transition
r = 0.54; % flowering to resprouting adult transition (Pardini 2008)
fr = 0.95; % resprout fertility reduction (Pardini 2008)
decay_yrs = 10; % seed bank degradation time
    b1 = 1-1/(decay_yrs*12)*2; % bank seed survival
    b2 = 1-1/(decay_yrs*12)*1; 
    b3 = 1-1/(decay_yrs*12)*1; 
    b4 = 1-1/(decay_yrs*12)*8;
B0f = 7.48933; % density dependence constants 
    B1f = -0.03893;
    B0s = 0.11635;
    B1s = -0.01612;
    B2s = -0.00144;
    B3s = -0.00092;
    B0w = 1.32702;
    B1w = -0.50269;

%% INITIAL CONDITIONS
S_new0 = 0; % new seeds
S_bank0 = 100; % bank seeds
Sg0 = 0; % seedings
R0 = 0; % rosettes
A_flower0 = 0; % flowering adults
A_fruit0 = 0; % fruiting adults
A_resprout0 = 0; % reprouting adults

n = [S_new0 S_bank0 Sg0 R0 A_flower0 A_fruit0 A_resprout0];

t_iter = 3000; % number of iterations (seasons) to run
t_rec = 300; % additional iterations to record

%% Management matrices
%%% Season 1
M1 = diag([1 1 1 1 1 1 1]);
M1(3,3) = (1 - h1); % spring herbicide on seedlings
M1(4,4) = (1 - h1); % spring herbicide on rosettes

%%% Season 2
M2 = diag([1 1 1 1 1 1 1]);
M2(5,5) = (1 - p2)*(1 - c2); % pulling, clipping flowering adults
M2(7,5) = c2*r; % resprouts from early clipping

%%% Season 3
M3 = diag([1 1 1 1 1 1 1]);
M3(6,6) = (1 - c3); % clipping fruiting adults

%%% Season 4
M4 = diag([1 1 1 1 1 1 1]);
M4(1,1) = (1-ws); % biocontrol on seeds
M4(4,4) = (1-max(0,(0.895*ws-0.008)))*(1-h4); % biocontrol, fall herbicide on rosettes
    
%% Run a year-long simulation
for t = 1:4:t_iter + t_rec 
    
    %%% Season 1 %%%
    n(t,:) = M1*n(t,:)'; % management for Season 1
    B1 = zeros(7); % projection matrix B1
    B1(4,3) = s1; B1(5,4) = s5; B1(2,2) = b1;
    Rt = n(t, 4);
    f = exp(B0f + B1f * Rt); % dd for fertility
    n(t+1,:) = B1*n(t,:)'; % transition to Season 2
    
    %%% Season 2 %%%
    n(t+1,:) = M2*n(t+1,:)'; % management for Season 2 
    Rt = n(t+1, 4); At = n(t+1, 5);
    s2 = 1/(1 + exp(-(B0s + B1s*At + B2s*Rt + B3s*At*Rt))); % dd for 1st yr rosette survival 
    B2 = zeros(7); % projection matrix B2
    B2(4,4) = s2; B2(6,5) = a; B2(2,2) = b2;
    n(t+2,:) = B2*n(t+1,:)'; % transition to Season 3

    %%% Season 3 %%%
    n(t+2,:) = M3*n(t+2,:)'; % management for Season 3
    B3 = zeros(7); % projection matrix B3
    B3(4,4) = s3; B3(1,6) = f; B3(1,7) = f*(1-fr); B3(2,2) = b3;
    n(t+3,:) = B3*n(t+2,:)'; % transition to Season 4
    
    %%% Season 4 %%%
    n(t+3,:) = M4*n(t+3,:)'; % management in Season 4
    Rt = n(t+3, 4);
        s4 = 1/(1 + exp(-(B0w + B1w * log(Rt + 1)))); % dd for 1st to 2nd yr rosette survival
    B4 = zeros(7); % projection matrix B4
        B4(4,4) = s4; B4(3,1) = v*g1; B4(2,1) = v*(1-g1); B4(3,2) = g2; B4(2,2) = b4*(1-g2);
    n(t+4,:) = B4*n(t+3,:)'; % transition to Season 1
    
end

%% Partition the entries of "n" into different stages.
% As in this example, we may only want to store densities after asymptotic
% dynamics have been reached (here, we are recording an additional t_rec
% seasons after the first t_iter seasons).
S_new_eq = n(t_iter+1:t_iter + t_rec,1);
S_bank_eq = n(t_iter+1:t_iter + t_rec,2);
Sg_eq = n(t_iter+1:t_iter + t_rec,3);
R_eq = n(t_iter+1:t_iter + t_rec,4);
A_flower_eq = n(t_iter+1:t_iter + t_rec,5);
A_fruit_eq = n(t_iter+1:t_iter + t_rec,6);
A_resprout_eq = n(t_iter+1:t_iter + t_rec,7);

%% Store seasonal densities for each stage separately.
seasons = 4; co1 = 1; co2 = 1; co3 = 1; co4 = 1;

for k = 1:length(S_new_eq)
    if mod(k,seasons) == 1
        store1(co1, 1) = S_new_eq(k); 
        store1(co1, 2) = S_bank_eq(k); 
        store1(co1, 3) = Sg_eq(k); 
        store1(co1, 4) = R_eq(k); 
        store1(co1, 5) = A_flower_eq(k);
        store1(co1, 6) = A_fruit_eq(k);
        store1(co1, 7) = A_resprout_eq(k);
        co1 = co1 + 1;
    end
    
    if mod(k,seasons) == 2
        store2(co2, 1) = S_new_eq(k); 
        store2(co2, 2) = S_bank_eq(k); 
        store2(co2, 3) = Sg_eq(k); 
        store2(co2, 4) = R_eq(k); 
        store2(co2, 5) = A_flower_eq(k);
        store2(co2, 6) = A_fruit_eq(k);
        store2(co2, 7) = A_resprout_eq(k);
        co2 = co2 + 1;
    end
    
    if mod(k,seasons) == 3
        store3(co3, 1) = S_new_eq(k); 
        store3(co3, 2) = S_bank_eq(k); 
        store3(co3, 3) = Sg_eq(k); 
        store3(co3, 4) = R_eq(k); 
        store3(co3, 5) = A_flower_eq(k);
        store3(co3, 6) = A_fruit_eq(k);
        store3(co3, 7) = A_resprout_eq(k);
        co3 = co3 + 1;
    end
    
     if mod(k,seasons) == 0
        store4(co4, 1) = S_new_eq(k); 
        store4(co4, 2) = S_bank_eq(k); 
        store4(co4, 3) = Sg_eq(k); 
        store4(co4, 4) = R_eq(k); 
        store4(co4, 5) = A_flower_eq(k);
        store4(co4, 6) = A_fruit_eq(k);
        store4(co4, 7) = A_resprout_eq(k);
        co4 = co4 + 1;
    end

end

% Find and return the unique seasonal densities for each stage in vectors
% eq1 (Season 1), eq2 (Season 2), eq3 (Season 3), eq4 (Season 4). We make
% use of functions unique_no_sort and padcat, which can downloaded from 
% the MATLAB File Exchange.

pl = 1000; % decimal place to which we would consider densities identical
% In this case, we will consider densities to be the same if they are within
% 1/1000 of each other

% Season 1
eq1 = padcat(unique_no_sort(round(pl*store1(:,1))/pl)', ...
                   unique_no_sort(round(pl*store1(:,2))/pl)', ...
                   unique_no_sort(round(pl*store1(:,3))/pl)', ...
                   unique_no_sort(round(pl*store1(:,4))/pl)', ...
                   unique_no_sort(round(pl*store1(:,5))/pl)', ...
                   unique_no_sort(round(pl*store1(:,6))/pl)', ...
                   unique_no_sort(round(pl*store1(:,7))/pl)');
               
eq1(isnan(eq1)) = 0;                

% Season 2               
eq2 = padcat(unique_no_sort(round(pl*store2(:,1))/pl)', ...
                   unique_no_sort(round(pl*store2(:,2))/pl)', ...
                   unique_no_sort(round(pl*store2(:,3))/pl)', ...
                   unique_no_sort(round(pl*store2(:,4))/pl)', ...
                   unique_no_sort(round(pl*store2(:,5))/pl)', ...
                   unique_no_sort(round(pl*store2(:,6))/pl)', ...
                   unique_no_sort(round(pl*store2(:,7))/pl)');
               
eq2(isnan(eq2)) = 0;                 

% Season 3               
eq3 = padcat(unique_no_sort(round(pl*store3(:,1))/pl)', ...
                   unique_no_sort(round(pl*store3(:,2))/pl)', ...
                   unique_no_sort(round(pl*store3(:,3))/pl)', ...
                   unique_no_sort(round(pl*store3(:,4))/pl)', ...
                   unique_no_sort(round(pl*store3(:,5))/pl)', ...
                   unique_no_sort(round(pl*store3(:,6))/pl)', ...
                   unique_no_sort(round(pl*store3(:,7))/pl)');
               
eq3(isnan(eq3)) = 0;  

% Season 4               
eq4 = padcat(unique_no_sort(round(pl*store4(:,1))/pl)', ...
                   unique_no_sort(round(pl*store4(:,2))/pl)', ...
                   unique_no_sort(round(pl*store4(:,3))/pl)', ...
                   unique_no_sort(round(pl*store4(:,4))/pl)', ...
                   unique_no_sort(round(pl*store4(:,5))/pl)', ...
                   unique_no_sort(round(pl*store4(:,6))/pl)', ...
                   unique_no_sort(round(pl*store4(:,7))/pl)');
               
eq4(isnan(eq4)) = 0;  