% Code to perform sensitivity analyses for the seasonal garlic mustard 
% model (Figure 4).

%% PARAMETERS
k = 8; % number of points in the seasonal cycle
s = 7; % number of stages

B0f = 7.48933; % density dependence constants
B1f = -0.03893;
B0s = 0.11635;
B1s = -0.01612;
B2s = -0.00144;
B3s = -0.00092;
B0w = 1.32702;
B1w = -0.50269;

%% VARIABLES
% Define any variables we want to take derivatives to symbolically using
% the MATLAB symbolic toolbox.

syms v g1 g2 s1 s3 s5 a r fr b1 b2 b3 b4 % demographic parameters
syms h1 p2 c2 c3 h4 ws  % management parameters
syms S_new S_bank Sg R A_flower A_fruit A_respout % stages 

%% PROJECTION MATRICES
% Write out the seasonal projection matrices and transition matrices in
% terms of the defined variables.

%% Seasonal projection matrices %%
B1 = [0 0 0 0 0 0 0; ...
    0 b1 0 0 0 0 0; ...
    0 0 0 0 0 0 0; ...
    0 0 s1 0 0 0 0; ...
    0 0 0 s5 0 0 0; ...
    0 0 0 0 0 0 0; ...
    0 0 0 0 0 0 0];

s2 = 1/(1 + exp(-(B0s + B1s*A_flower + B2s*R + B3s*A_flower*R)));

B2 = [0 0 0 0 0 0 0; ...
    0 b2 0 0 0 0 0; ...
    0 0 0 0 0 0 0; ...
    0 0 0 s2 0 0 0; ...
    0 0 0 0 0 0 0; ...
    0 0 0 0 a 0 0; ...
    0 0 0 0 0 0 0];

f = exp(B0f + B1f * R);

B3 = [0 0 0 0 0 f f*(1-fr); ...
    0 b3 0 0 0 0 0; ...
    0 0 0 0 0 0 0; ...
    0 0 0 s3 0 0 0; ...
    0 0 0 0 0 0 0; ...
    0 0 0 0 0 0 0; ...
    0 0 0 0 0 0 0];

s4 = 1/(1 + exp(-(B0w + B1w * log(R + 1))));

B4 = [0 0 0 0 0 0 0; ...
    v*(1-g1) b4*(1-g2) 0 0 0 0 0; ...
    v*g1 g2 0 0 0 0 0; ...
    0 0 0 s4 0 0 0; ...
    0 0 0 0 0 0 0; ...
    0 0 0 0 0 0 0; ...
    0 0 0 0 0 0 0];

%% Management matrices %%
% Season 1
M1 = [1 0 0 0 0 0 0; ...
    0 1 0 0 0 0 0; ...
    0 0 (1 - h1) 0 0 0 0; ...
    0 0 0 (1 - h1) 0 0 0; ...
    0 0 0 0 1 0 0; ...
    0 0 0 0 0 1 0; ...
    0 0 0 0 0 0 1];

% Season 2
M2 = [1 0 0 0 0 0 0; ...
    0 1 0 0 0 0 0; ...
    0 0 1 0 0 0 0; ...
    0 0 0 1 0 0 0; ...
    0 0 0 0 (1-p2)*(1-c2) 0 0; ...
    0 0 0 0 0 1 0; ...
    0 0 0 0 c2*r 0 1];

% Season 3
M3 = [1 0 0 0 0 0 0; ...
    0 1 0 0 0 0 0; ...
    0 0 1 0 0 0 0; ...
    0 0 0 1 0 0 0; ...
    0 0 0 0 1 0 0; ...
    0 0 0 0 0 (1-c3) 0; ...
    0 0 0 0 0 0 1];

% Season 4
M4 = [(1-ws) 0 0 0 0 0 0; ...
    0 1 0 0 0 0 0; ...
    0 0 1 0 0 0 0; ...
    0 0 0 (1-((0.895*ws-0.008)))*(1-h4) 0 0 0; ...
    0 0 0 0 1 0 0; ...
    0 0 0 0 0 1 0; ...
    0 0 0 0 0 0 1];

C1 = M2*B1; % product matrices from (20)
C2 = M3*B2;
C3 = M4*B3;
C4 = M1*B4;

%% MATRIX DERIVATIVES
n_param = {'S_new', 'S_bank', 'Sg', 'R', 'A_flower', 'A_fruit', 'A_resprout'}; % entries of population vector n
theta_param = {'v', 'g1', 'g2', 's1', 's3', 's5', 'a', 'r', 'fr', 'b1', 'b2', 'b3', 'b4', 'h1', 'p2', 'c2', 'c3', 'h4', 'ws'}; % entries of parameter vector theta

% get symbolic derivatives for (26)
dC_dn.C1 = []; % dvecC1/dn
dC_dn.C2 = []; % dvecC2/dn
dC_dn.C3 = []; % dvecC3/dn
dC_dn.C4 = []; % dvecC4/dn

for i = 1:length(n_param)
    n_paramcurr = n_param{i};
    
    dC1_dn = diff(C1, n_paramcurr);
    dC_dn.C1 = [dC_dn.C1 dC1_dn(:)]; 

    dC2_dn = diff(C2, n_paramcurr);
    dC_dn.C2 = [dC_dn.C2 dC2_dn(:)]; 
    
    dC3_dn = diff(C3, n_paramcurr);
    dC_dn.C3 = [dC_dn.C3 dC3_dn(:)]; 
    
    dC4_dn = diff(C4, n_paramcurr);
    dC_dn.C4 = [dC_dn.C4 dC4_dn(:)]; 
end

% get symbolic derivatives for for (27)
dC_dtheta.C1 = []; % dvecC1/dtheta
dC_dtheta.C2 = []; % dvecC2/dtheta
dC_dtheta.C3 = []; % dvecC3/dtheta
dC_dtheta.C4 = []; % dvecC4/dtheta

for i = 1:length(theta_param)    
    theta_paramcurr = theta_param{i};
    
    dC1_dtheta = diff(C1, theta_paramcurr);
    dC_dtheta.C1 = [dC_dtheta.C1 dC1_dtheta(:)];
    
    dC2_dtheta = diff(C2, theta_paramcurr);
    dC_dtheta.C2 = [dC_dtheta.C2 dC2_dtheta(:)];
    
    dC3_dtheta = diff(C3, theta_paramcurr);
    dC_dtheta.C3 = [dC_dtheta.C3 dC3_dtheta(:)];
    
    dC4_dtheta = diff(C4, theta_paramcurr);
    dC_dtheta.C4 = [dC_dtheta.C4 dC4_dtheta(:)];
end

%% VARIABLE VALUES
% Now that derivatives have been taken symbolically, we want to evaluate at
% them at the parameter values in the model.

% If we are calculating sensitivities for an initially-unmanaged 
% population, all management parameters are evaluated at 0.
h1 = 0; % spring herbicide
p2 = 0; % pulling
c2 = 0; % early clipping
c3 = 0; % late clipping
h4 = 0; % fall herbicide
ws = 0; % biocontrol (seed effect)
wr = max(0, (0.895*ws-0.008)); % biocontrol (rosette effect)

% Demographic parameter values are the same as in gm_base.m (Table 1)
v = 0.8228; % new seed survival
g1 = 0.55034; % new seed germination
g2 = 0.31705; % bank seed germination
s1 = 0.131; % seedling to 1st year rosette transition
s3 = 1; % 1st year rosette survival
s5 = 1; % 2nd year rosette to flowering adult transition
a = 1; % flowering to fruiting adult transition
r = 0.54; % flowering to resprouting adult transition (Pardini 2008)
fr = 0.95; % resprout fertility reduction (Pardini 2008)
decay_yrs = 10; % seed bank degradation time
    b1 = 1-1/(decay_yrs*12)*2; % bank seed survival
    b2 = 1-1/(decay_yrs*12)*1; 
    b3 = 1-1/(decay_yrs*12)*1; 
    b4 = 1-1/(decay_yrs*12)*8;

%% 8-cycle dynamics
% In this case, we are looking at an initially unmanaged population, which
% has a biennial seasonal 8-cycle. We thus use the 8 seasonal population
% vectors from (19).

[eqS1 eqS2 eqS3 eqS4] = gm_eq(h1, p2, c2, c3, h4, ws); 
n1 = eqS1(1,:); 
n2 = eqS2(1,:);
n3 = eqS3(1,:);
n4 = eqS4(1,:);
n5 = eqS1(2,:);
n6 = eqS2(2,:);
n7 = eqS3(2,:);
n8 = eqS4(2,:);
% The function gm_eq is a slighly modified version of gm_base.m that 
% returns all the seasonal population vectors in the stable 8-cycle, for 
% the given values of management parameters (in this case, all 0). Note
% that high levels of management may collapse the 8-cycle into a 4-cycle,
% upon which the following code should be modified accordingly.
%   

% Now evaluate the matrices and derivatives using the appropriate seasonal
% population vectors.

% C1 and C5 are density-independent
C1_e = eval(C1);
C1_dtheta = eval(dC_dtheta.C1);

C5_e = eval(C1); % C5 is the same as C1 due to density-independence
C5_dtheta = eval(dC_dtheta.C1);

% Densities in n1 determine C3 (19) and all its derivatives
S_new = n1(1); S_bank = n1(2); Sg = n1(3); R = n1(4);
A_flower = n1(5); A_fruit = n1(6); A_resprout = n1(7);

C3_e = eval(C3);
C3_dtheta = eval(dC_dtheta.C3);
C3_dn1 = eval(dC_dn.C3);

% Densities in n2 determine C2 and all its derivatives
S_new = n2(1); S_bank = n2(2); Sg = n2(3); R = n2(4);
A_flower = n2(5); A_fruit = n2(6); A_resprout = n2(7);

C2_e = eval(C2);
C2_dtheta = eval(dC_dtheta.C2);
C2_dn2 = eval(dC_dn.C2);

% Densities in n4 determine C4 and all its derivatives
S_new = n4(1); S_bank = n4(2); Sg = n4(3); R = n4(4);
A_flower = n4(5); A_fruit = n4(6); A_resprout = n4(7);

C4_e = eval(C4);
C4_dtheta = eval(dC_dtheta.C4);
C4_dn4 = eval(dC_dn.C4);

% Densities in n5 determine C7 and all its derivatives
S_new = n5(1); S_bank = n5(2); Sg = n5(3); R = n5(4);
A_flower = n5(5); A_fruit = n5(6); A_resprout = n5(7);

C7_e = eval(C3); % note C7 has the same form as C3, but is evaluated at n5 rather than n1
C7_dtheta = eval(dC_dtheta.C3);
C7_dn5 = eval(dC_dn.C3);

% Densities in n6 determine C6 and all its derivatives
S_new = n6(1); S_bank = n6(2); Sg = n6(3); R = n6(4);
A_flower = n6(5); A_fruit = n6(6); A_resprout = n6(7);

C6_e = eval(C2); % C6 has the same form as C2, but is evaluated at n6 rather than n2
C6_dtheta = eval(dC_dtheta.C2);
C6_dn6 = eval(dC_dn.C2);

% Densities in n8 determine C8 and all its derivatives
S_new = n8(1); S_bank = n8(2); Sg = n8(3); R = n8(4);
A_flower = n8(5); A_fruit = n8(6); A_resprout = n8(7);

C8_e = eval(C4); % C8 has the same form as C4, but is evaluated at n8 rather than n4
C8_dtheta = eval(dC_dtheta.C4);
C8_dn8 = eval(dC_dn.C4);

%% Block matrices
% Now form the block matrices that will allow us to evaluate (28).

nblock = vertcat(n1', n2', n3', n4', n5', n5', n7', n8'); % forms (22), but with 8 vectors

Bblock1 = blkdiag(C1_e, C2_e, C3_e, C4_e, C5_e, C6_e, C7_e); 
Bblock = rev_blkdiag(Bblock1, C8_e); 

Is = eye(s); % s x s identity matrix
H1 = kron(n1, Is); H2 = kron(n2, Is); H3 = kron(n3, Is); H4 = kron(n4, Is);
H5 = kron(n5, Is); H6 = kron(n6, Is); H7 = kron(n7, Is); H8 = kron(n8, Is);
Hblock1 = blkdiag(H1, H2, H3, H4, H5, H6, H7); 
Hblock = rev_blkdiag(Hblock1, H8); % (25)

mat2 = zeros(8,8); mat2(2,2) = 1;
mat3 = zeros(8,8); mat3(3,1) = 1;
mat4 = zeros(8,8); mat4(4,4) = 1;
mat6 = zeros(8,8); mat6(6,6) = 1;
mat7 = zeros(8,8); mat7(7,5) = 1;
mat8 = zeros(8,8); mat8(8,8) = 1;
Cblock = kron(mat2, C2_dn2) + kron(mat3, C3_dn1) + kron(mat4, C4_dn4) + kron(mat6, C6_dn6) + kron(mat7, C7_dn5) + kron(mat8, C8_dn8); % (26)

Dblock = vertcat(C1_dtheta, C2_dtheta, C3_dtheta, C4_dtheta, C5_dtheta, C6_dtheta, C7_dtheta, C8_dtheta); % (27)

Iblock = eye(k*s); % ks x ks identity matrix

dn_dpblock = inv(Iblock - Bblock - Hblock*Cblock)*Hblock*Dblock; % sensitivity equation (28)

%% Take stage or time averages if desired.
Ik = eye(k);
Is = eye(s);  
c = [0 0 0 1 1 1 1]'; % weights for each stage
b = [2/24 1/24 1/24 8/24 2/24 1/24 1/24 8/24]'; % proportion of time spent at each cycle point
    
d_stageavg_dp = kron(Ik, c') * dn_dpblock; % (28) 
d_timeavg_dp = kron(b', Is) * dn_dpblock; % (29)
d_stagetimeavg_dp = kron(b', c') * dn_dpblock; % (30) 